import { FC, useCallback, useState } from 'react'
import AnimatedPageContainer from '../AnimatedPageContainer';
import { TypeEditor } from './type-editor';
import { useObjectState, useObjectStateWrapper } from '../../hooks/useObjectState';
import { Schema, TypeType } from '../../types/schema';
import { useInput } from '../../hooks/useInput';
import { useRecoilState } from 'recoil';
import { SchemaEditAtom } from '../../recoil/atoms/schema';


export const SchemaBuilder: FC = () => {
  const [schema, setSchema] = useRecoilState(SchemaEditAtom);
  const {update: updateSchema} = useObjectStateWrapper<Schema>(schema, setSchema);

  const {value: typeName, bind: bindTypeName, reset: resetTypeName} = useInput('');

  const [pageNumber, setPageNumber] = useState(1);

  const saveType = useCallback((name: string, type: TypeType) => {
    updateSchema(e => ({
      types: {
        ...e.types,
      }
    }));
    resetTypeName();
  }, [resetTypeName, updateSchema]);
  
  return (
    <div className="container flex gap-4 p-8">
      <div className="panel w-2/3 h-full">
        <AnimatedPageContainer currentPage={pageNumber}>
          <div>
            <p className="subheader">Add A Type</p>
            <input type="text" {...bindTypeName} />
            <button className="interactive" disabled={!typeName} onClick={() => setPageNumber(1)}>Configure</button>
          </div>
          <TypeEditor name={typeName} saveType={saveType} />
        </AnimatedPageContainer>
      </div>
      <div className="panel w-1/3 whitespace-pre-wrap">
        {JSON.stringify(schema, null, 2)}
      </div>
    </div>
  )
}